package org.etsi.its.adapter;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import de.tu_berlin.cs.uebb.muttcn.runtime.RB;
import de.tu_berlin.cs.uebb.ttcn.runtime.OctetString;

public class TraceParser {
	private List<List<TransactionStructure>> traceList = new ArrayList<List<TransactionStructure>>();
	private String startCommandStr = ":  trace";
	private String falseCommandStr = ": trace c";
	private String stopCommandStr = "Those were the most recent";
	
	private String filePath;


	private final RB RB;
	
	public TraceParser(RB RB, String filePath){
		this.RB = RB;
		this.filePath = filePath;
	}
	
//	public static void main(String[] args) {
//		startTraceParsing("C:\\Demo_Dump2.txt");
//		
//	}
	
	public void parseTrace(){
		try {
			ArrayList<TransactionStructure> transactions = null;
			
			FileReader fr = new FileReader(filePath);
			BufferedReader br = new BufferedReader(fr);
			String line=null;
			boolean transactionStarted=false;
			while(null!=(line=br.readLine())) {
//				if (line.startsWith("here the dump starts ---------->")) {
				if (line.startsWith(startCommandStr) && !line.startsWith(falseCommandStr)){
					transactionStarted = true;
					transactions = new ArrayList<TransactionStructure>();
					traceList.add(transactions);
					System.out.println("++++++++++++++++ Trace started ++++++++++++++++");
				}
//				else if (line.startsWith("<-----------")) {
				else if (line.startsWith(stopCommandStr)) {
					transactionStarted = false;
					System.out.println("++++++++++++++++ Trace finished ++++++++++++++++");
				}
				if (transactionStarted){
					if (line.startsWith("0x")) {
						line = line.substring(2); //cut 0x
						TransactionStructure transaction = new TransactionStructure();
						line = line.trim();
						String timestamp=line.substring(0, line.indexOf(" "));
						line=line.substring(line.indexOf(" "));
						
						System.out.println("Timestamp:   " + timestamp);
						transaction.setTimeStamp(new BigInteger(timestamp, 16));
						line = line.substring(line.lastIndexOf("]")+1).trim();
						StringTokenizer st = new StringTokenizer(line, " ");
						int tokens = st.countTokens();
						int bytes=0;
						int bytesCounter=0;
						for (int i=0; i<tokens;i++) {
							String token = st.nextToken();
							switch (i) {
							case 0:
								if (token.equals("UL")) { //UL
									transaction.setDirection(TransactionStructure.RSU); 
								}
								else if(token.equals("DL")) { //DL
									transaction.setDirection(TransactionStructure.OBU); 
								}
								else {
									//TODO error!
								}
								System.out.println("Direction:   " + token);
								break;
							case 1:
								System.out.println("no of bytes: " + token);
								bytes = Integer.parseInt(token);
								transaction.setNoOfBytes(bytes);
								break;
							case 2:
								//ignore
								break;
							case 3:
								if (bytesCounter<bytes) {
									System.out.print  ("payload    : " + token);
									transaction.setPayload(new OctetString(RB, token).toByteArray());
									
									bytesCounter++;
								}
								break;
							default:
								if (bytesCounter<bytes) {
									System.out.print(token);
									byte [] oldPayload = transaction.getPayload();
									transaction.setPayload(new OctetString(RB, oldPayload.length, oldPayload).concat(new OctetString(RB, token)).toByteArray());
									bytesCounter++;
								}
								if (i==tokens-1) {
									System.out.println();
								}
								break;
							}
						}
						transactions.add(transaction);
					}
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	public int getTraceCount(){
		return traceList.size();
	}

	public List<TransactionStructure> getTrace(int index) {
		return traceList.get(index);
	}

}
