package org.etsi.its.adapter;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.etsi.ttcn.tci.BooleanValue;
import org.etsi.ttcn.tci.CharstringValue;
import org.etsi.ttcn.tci.IntegerValue;
import org.etsi.ttcn.tri.TriActionTemplate;
import org.etsi.ttcn.tri.TriAddress;
import org.etsi.ttcn.tri.TriAddressList;
import org.etsi.ttcn.tri.TriComponentId;
import org.etsi.ttcn.tri.TriException;
import org.etsi.ttcn.tri.TriMessage;
import org.etsi.ttcn.tri.TriParameter;
import org.etsi.ttcn.tri.TriParameterList;
import org.etsi.ttcn.tri.TriParameterPassingMode;
import org.etsi.ttcn.tri.TriPortId;
import org.etsi.ttcn.tri.TriPortIdList;
import org.etsi.ttcn.tri.TriSignatureId;
import org.etsi.ttcn.tri.TriStatus;
import org.etsi.ttcn.tri.TriTestCaseId;

import com.testingtech.ttcn.tri.AbstractPlugin;
import com.testingtech.ttcn.tri.ISAPlugin;
import com.testingtech.ttcn.tri.PluginIdentifier;
import com.testingtech.ttcn.tri.TriAddressImpl;
import com.testingtech.ttcn.tri.TriMessageImpl;
import com.testingtech.ttcn.tri.TriStatusImpl;

import de.tu_berlin.cs.uebb.muttcn.runtime.RB;
import de.tu_berlin.cs.uebb.ttcn.runtime.LocalMessage;
import de.tu_berlin.cs.uebb.ttcn.tri.TriMessageParameter;

public class TracePortPlugin extends AbstractPlugin implements  ISAPlugin {

	private static final String MSG_ENCODING = "BaseMessages";
	private static final String SIGNATURE_ENCODING = "Signatures";
	private static final long serialVersionUID = 7436181385675637901L;
	private String tracePath = "";
	private boolean isFolder = false;
	private List<String> traceNameList = new ArrayList<String>();
	private RB RB;
	private PluginIdentifier pluginId;
	private TraceParser traceParse;
	private TriComponentId ulComponent;

	private TriPortId ulTsiPortId;
	private TriComponentId dlComponent;
	private TriPortId dlTsiPortId;
	
	public TracePortPlugin(PluginIdentifier pluginId){
		this.pluginId = pluginId;
	}


	public void setRB(RB rb) {
		RB = rb;
	}

	public TriStatus setUp() {
		return new TriStatusImpl();
	}

	public TriStatus tearDown() {
		return new TriStatusImpl();
	}

	public TriStatus triCall(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId, TriParameterList parameterList) {
		
		String signatureName = signatureId.getSignatureName();
		if ("setTraceFile".equals(signatureName)) {
			TriParameter tracePathParam = parameterList.get(0);
			String  tracePath = getCharstringParameterData(tracePathParam);
			boolean result = setTraceFile(tracePath);
			BooleanReturn(componentId, portId, address, signatureId,
					parameterList, result);
		}else if("isFolder".equals(signatureName)){
			BooleanReturn(componentId, portId, address, signatureId, parameterList, isFolder);
		}else if("getTransactionCnt".equals(signatureName)){
			int traceCnt = getTraceCnt();
			IntegerReturn(componentId, portId, address, signatureId,
					parameterList, traceCnt);
		}else if("startAnalyzeTrace".equals(signatureName)){
			TriParameter transactionCntParam = parameterList.get(0);
			final int transactionCnt = getIntegerParameterData(transactionCntParam);
			new Thread(tracePath) {

				public void run() {
					startAnalyzeTrace(transactionCnt);
				}
				
			}.start();
			BooleanReturn(componentId, portId, address, signatureId,
					parameterList, true);
		}else if("finishAnalyzeTrace".equals(signatureName)){
			//TOD0
		}
		
		
		return new TriStatusImpl();
	}


	private void IntegerReturn(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId,
			TriParameterList parameterList, int traceCnt) {
		IntegerValue iv = (IntegerValue) RB.getTciCDRequired().getInteger().newInstance();
		iv.setInt(traceCnt);
		TriMessage returnValue = RB.TestAdapter.getCodec(SIGNATURE_ENCODING).encode(iv);
		RB.getTriCommunicationTE().triEnqueueReply(portId, address, componentId, signatureId, parameterList, new TriMessageParameter(RB, TriParameterPassingMode.TRI_IN, "", returnValue));
	}


	private void BooleanReturn(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId,
			TriParameterList parameterList, boolean result) {
		BooleanValue bv = (BooleanValue) RB.getTciCDRequired().getBoolean().newInstance();
		bv.setBoolean(result);
		TriMessage returnValue = RB.TestAdapter.getCodec(SIGNATURE_ENCODING).encode(bv);
		RB.getTriCommunicationTE().triEnqueueReply(portId, address, componentId, signatureId, parameterList, new TriMessageParameter(RB, TriParameterPassingMode.TRI_IN, "", returnValue));
	}

	private boolean setTraceFile(String tracePathPr){
		File traceFile = new File(tracePathPr);
		
		try {
			boolean fileCreated = traceFile.createNewFile();
			if (!fileCreated && traceFile.canRead()) {
				this.tracePath = tracePathPr;
				return true;
			}
			if (fileCreated) {
				traceFile.delete();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
//	private boolean setTraceFile(String tracePathPr){
//		File traceFile = new File(tracePathPr);
//		
//		if (traceFile.exists() && traceFile.canRead()) {
//			this.tracePath = tracePathPr;
//			isFolder(traceFile);
//			return true;
//		}
//
//		return false;
//	}
	

	private void isFolder(File traceFile){
		 isFolder =  traceFile.isDirectory();
	}
	
	private void getFolderContent(File folder){
		traceNameList =  Arrays.asList(folder.list());

	}
	
	private int getTraceCnt(){	
		traceParse = new TraceParser(RB, tracePath);
		traceParse.parseTrace();
		
		return traceParse.getTraceCount();

	}
	
	private boolean startAnalyzeTrace(int trace){
        boolean result = false;
		List<TransactionStructure> transactions = traceParse.getTrace(trace);
		TriPortId tsiPortId = null;
		TriComponentId componentId = null;
		
		for (TransactionStructure transaction : transactions) {
			if (transaction.getDirection() == TransactionStructure.RSU) { //UL
				tsiPortId = ulTsiPortId;
				componentId = ulComponent;
			}
			if (transaction.getDirection() == TransactionStructure.OBU) { //DL
				tsiPortId = dlTsiPortId;
				componentId = dlComponent;
			}
			IntegerValue iv = (IntegerValue) RB.getTciCDRequired().getInteger().newInstance();
			iv.setBigInt(transaction.getTimeStamp());
			TriMessage address = RB.TestAdapter.getCodec(MSG_ENCODING).encode(iv);
			byte[] payload = transaction.getPayload();
			if (payload==null) {
				payload = new byte[]{};//TODO for empty messages it is null instead of empty array
			}
			RB.getTriCommunicationTE().triEnqueueMsg(tsiPortId, new TriAddressImpl(address.getEncodedMessage()), componentId, new TriMessageImpl(payload));
		}
		return result;
	}
	
	public String getCharstringParameterData(final TriParameter param) throws UnsupportedOperationException {
		if (param instanceof TriMessageParameter) {
			TriMessageParameter triMsgParam = (TriMessageParameter) param;
			if (triMsgParam.getMessage() instanceof LocalMessage) {
				LocalMessage localMsg = (LocalMessage) triMsgParam.getMessage();
				if (localMsg.getContent() instanceof CharstringValue) {
					CharstringValue v = (CharstringValue) localMsg.getContent();
					if (!v.notPresent()) {
						return v.getString();
					}
					else {
						return null;
					}
				}
			}
		}
		throw new UnsupportedOperationException("getCharstringParameterData: Parameter not of type charstring");
	}
	
	public int getIntegerParameterData(final TriParameter param) throws UnsupportedOperationException {
		if (param instanceof TriMessageParameter) {
			TriMessageParameter triMsgParam = (TriMessageParameter) param;
			if (triMsgParam.getMessage() instanceof LocalMessage) {
				LocalMessage localMsg = (LocalMessage) triMsgParam.getMessage();
				if (localMsg.getContent() instanceof IntegerValue) {
					IntegerValue v = (IntegerValue) localMsg.getContent();
					if (!v.notPresent()) {
						return v.getInt();
					}
				}
			}
		}
		throw new UnsupportedOperationException("getIntegerParameterData: Parameter not of type integer or omit");

	}

	public TriStatus triCallBC(TriComponentId componentId, TriPortId portId,
			TriSignatureId signatureId, TriParameterList parameterList) {
		return new TriStatusImpl("triCallBC not supported");
	}


	public TriStatus triCallMC(TriComponentId componentId, TriPortId portId,
			TriAddressList addressList, TriSignatureId signatureId, TriParameterList parameterList) {
		return new TriStatusImpl("triCallMC not supported");
	}


	public TriStatus triEndTestCase() {
		return new TriStatusImpl();
	}


	public TriStatus triExecuteTestcase(TriTestCaseId testCaseId, TriPortIdList portIdList) {
		return new TriStatusImpl();
	}


	public TriStatus triMap(TriPortId compPortId, TriPortId tsiPortId) {
		if (compPortId.getPortName().equals("rsuPort")) {
			ulComponent = compPortId.getComponent();
			ulTsiPortId = tsiPortId;
		}
		else if (compPortId.getPortName().equals("obuPort")) {
			dlComponent = compPortId.getComponent();
			dlTsiPortId = tsiPortId;
		}
		else if (compPortId.getPortName().equals("controlPort")) {
			
		}
		else {
			return new TriStatusImpl("Unsupported port: " + compPortId.getPortName());
		}
		return new TriStatusImpl();
	}


	public TriStatus triRaise(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId, TriException exception) {
		return new TriStatusImpl("triRaise not supported");
	}


	public TriStatus triRaiseBC(TriComponentId componentId, TriPortId portId,
			TriSignatureId signatureId, TriException exception) {
		return new TriStatusImpl("triRaiseBC not supported");
	}


	public TriStatus triRaiseMC(TriComponentId componentId, TriPortId portId,
			TriAddressList addressList, TriSignatureId signatureId, TriException exception) {
		return new TriStatusImpl("triRaiseMC not supported");
	}


	public TriStatus triReply(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriSignatureId signatureId, TriParameterList parameterList,
			TriParameter parameter) {
		return new TriStatusImpl("triReply not supported");
	}


	public TriStatus triReplyBC(TriComponentId componentId, TriPortId portId,
			TriSignatureId signatureId, TriParameterList parameterList, TriParameter parameter) {
		return new TriStatusImpl("triReplyBC not supported");
	}


	public TriStatus triReplyMC(TriComponentId componentId, TriPortId portId,
			TriAddressList addressList, TriSignatureId signatureId, TriParameterList parameterList,
			TriParameter parameter) {
		return new TriStatusImpl("triReplyMC not supported");
	}


	public TriStatus triSAReset() {
		return new TriStatusImpl();
	}


	public TriStatus triSend(TriComponentId componentId, TriPortId portId,
			TriAddress address, TriMessage message) {
		return new TriStatusImpl("triSend not supported");
	}


	public TriStatus triSendBC(TriComponentId componentId, TriPortId portId,
			TriMessage message) {
		return new TriStatusImpl("triSendBC not supported");
	}


	public TriStatus triSendMC(TriComponentId componentId, TriPortId portId,
			TriAddressList addressList, TriMessage message) {
		return new TriStatusImpl("triSendMC not supported");
	}


	public TriStatus triSutActionInformal(String str) {
		return new TriStatusImpl("triSutActionInformal not supported");
	}


	public TriStatus triSutActionTemplate(TriActionTemplate actionTemplate) {
		return new TriStatusImpl("triSutActionTemplate not supported");
	}


	public TriStatus triUnmap(TriPortId portId, TriPortId portId1) {
		ulComponent = null;
		ulTsiPortId = null;
		dlComponent = null;
		dlTsiPortId = null;
		return new TriStatusImpl();
	}

}
