package org.etsi.its.codec;

import org.etsi.ttcn.tci.RecordValue;
import org.etsi.ttcn.tci.Type;
import org.etsi.ttcn.tci.UnionValue;
import org.etsi.ttcn.tci.Value;
import org.etsi.ttcn.tri.TriMessage;

import com.testingtech.ttcn.logging.RTLoggingConstants;
import com.testingtech.ttcn.tci.codec.base.AbstractBaseCodec;
import com.testingtech.ttcn.tri.ICDPlugin;

import de.tu_berlin.cs.uebb.muttcn.runtime.RB;

public class TraceCodec extends AbstractBaseCodec implements ICDPlugin {

	public TraceCodec(RB rb) {
		super(rb);
	}

	public void setRB(RB rb) {
		RB = rb;
	}

	public boolean setUp() {
		return true;
	}

	public void tearDown() {
	}

	public synchronized Value decode(TriMessage message, Type decodingHypothesis) {
		bitpos=0;
		encodedMsg = message.getEncodedMessage();
		if (!decodingHypothesis.getName().equals("PayloadStructure")) {
			RB.getTciTLProvidedV321TT().tliRT("", System.nanoTime(), "", -1, null, RTLoggingConstants.RT_LOG_ERROR, "Only PayloadStructure type support by " + this.getClass().getName());
			return null;
		}
		UnionValue uv = (UnionValue) decodingHypothesis.newInstance();
		
		if (encodedMsg.length==0) { //empty message -> link message
			RecordValue rv = (RecordValue) uv.getVariant("linkMsg").getType().newInstance();
			String[] fieldNames = rv.getFieldNames();
			for (String fieldName : fieldNames) {
				rv.setFieldOmitted(fieldName);
			}
			uv.setVariant("linkMsg", rv);
		}
		else {
			if (encodedMsg[0]==(byte)0xFF) {
				//we have a broadcast message
				RecordValue rv = (RecordValue) uv.getVariant("broadcastMsg").getType().newInstance();
				int byteCnt=0;
				//broadcast
				byteCnt+=1;
				if (encodedMsg.length>=byteCnt) {
					rv.setField("broadcast", createOctetstringValue(encodedMsg, 1<<3));
				}
				//msg
				if (encodedMsg.length>byteCnt) {
					rv.setField("msg", createOctetstringValue(encodedMsg, (encodedMsg.length-byteCnt)<<3));
				}
				uv.setVariant("broadcastMsg", rv);
			}
			else {
				//we have a link message
				RecordValue rv = (RecordValue) uv.getVariant("linkMsg").getType().newInstance();
				
				int byteCnt=0;
				//linkId
				byteCnt+=4;
				if (encodedMsg.length>=byteCnt) {
					rv.setField("linkId", createOctetstringValue(encodedMsg, 4<<3));
				}
				//macControl
				byteCnt+=1;
				if (encodedMsg.length>=byteCnt) {
					rv.setField("macControl", createOctetstringValue(encodedMsg, 1<<3));
				}
				//linkControl
				byteCnt+=1;
				if (encodedMsg.length>=byteCnt) {
					rv.setField("linkControl", createOctetstringValue(encodedMsg, 1<<3));
				}
				//fragHeader
				byteCnt+=1;
				if (encodedMsg.length>=byteCnt) {
					rv.setField("fragHeader", createOctetstringValue(encodedMsg, 1<<3));
				}
				//tapdu
				if (encodedMsg.length>byteCnt) {
					rv.setField("tapdu", createOctetstringValue(encodedMsg, (encodedMsg.length-byteCnt)<<3));
				}
				
				uv.setVariant("linkMsg", rv);
			}
		}
		
		return uv;
	}

	public synchronized TriMessage encode(Value value) {
		return super.encode(value);
	}

}
